//-----------------------------------------------------------------------------
#ifndef CHUNK_LAYOUTS_H
#if !defined(DOXYGEN_SHOULD_SKIP_THIS) && !defined(WRAP_ANY)
#   define CHUNK_LAYOUTS_H CHUNK_LAYOUTS_H
#endif // DOXYGEN_SHOULD_SKIP_THIS && WRAP_ANY
//-----------------------------------------------------------------------------

// supported operating systems
#define osMSWin32      0
#define osLinux32      1
#define osMSWin64      2
#define osLinux64     3
#define osMSWinCE      4

// supported compilers
#define cVisualC   0
#define cBorlandC  1
#define cGCC       2

#if defined(WIN32) || defined(_WIN32) || defined(__WIN32__)
#   if defined(_WIN64) || defined(WIN64) || defined(__WIN64__)
#       define detectedOperatingSystem osMSWin64
#   else
#       if defined(_WIN32_WCE)
#           define detectedOperatingSystem osMSWinCE
#       else
#           define detectedOperatingSystem osMSWin32
#       endif // if defined(WIN32_PLATFORM_PSPC)
#   endif // if defined(_WIN64) || defined(WIN64) || defined(__WIN64__)
#   if defined(_MSC_VER)
#       define detectedCompiler cVisualC
#   elif defined(__BORLANDC__) || defined(__BCPLUSPLUS__)
#       define detectedCompiler cBorlandC
#   else
#       error "unsupported compiler"
#   endif
#elif defined(linux) || defined(__linux) || defined(__linux__)
#   if defined(__x86_64__) || defined(__powerpc64__) // -m64 makes GCC define __powerpc64__
#       define detectedOperatingSystem osLinux64
#   elif defined(__i386__) || defined(__arm__) || defined(__powerpc__) // and -m32 __powerpc__
#       define detectedOperatingSystem osLinux32
#   endif
#   ifdef __GNUC__
#       define detectedCompiler cGCC
#   else
#       error "unsupported compiler"
#   endif
#endif // #if defined(WIN32) || defined(_WIN32) || defined(__WIN32__)

#if !defined(detectedOperatingSystem) || !defined(detectedCompiler)
#   error "unsupported target environment"
#endif

#if(detectedCompiler == cGCC)
#   include <unistd.h>
#   include <stdint.h>      // needed for intXX_t
typedef int64_t         mv_int64;
typedef int32_t         mv_int32;
typedef int16_t         mv_int16;
typedef int8_t          mv_int8;
typedef uint64_t        mv_uint64;
typedef uint32_t        mv_uint32;
typedef uint16_t        mv_uint16;
typedef uint8_t         mv_uint8;
#elif( ( detectedCompiler == cVisualC ) || ( detectedCompiler == cBorlandC ) )
typedef __int64             mv_int64;
typedef __int32             mv_int32;
typedef __int16             mv_int16;
typedef __int8              mv_int8;
typedef unsigned __int64    mv_uint64;
typedef unsigned __int32    mv_uint32;
typedef unsigned __int16    mv_uint16;
typedef unsigned __int8     mv_uint8;
#endif

#if defined(linux) || defined(__linux) || defined(__linux__)
#   include <endian.h>
#endif // #if defined(linux) || defined(__linux) || defined(__linux__)

#define SWAP_64(l) \
    (((a & 0x00000000000000ffULL) << 56) | \
     ((a & 0x000000000000ff00ULL) << 40) | \
     ((a & 0x0000000000ff0000ULL) << 24) | \
     ((a & 0x00000000ff000000ULL) <<  8) | \
     ((a & 0x000000ff00000000ULL) >>  8) | \
     ((a & 0x0000ff0000000000ULL) >> 24) | \
     ((a & 0x00ff000000000000ULL) >> 40) | \
     ((a & 0xff00000000000000ULL) >> 56))

#define SWAP_32(l) \
    ((((l) & 0xff000000) >> 24) | \
     (((l) & 0x00ff0000) >> 8)  | \
     (((l) & 0x0000ff00) << 8)  | \
     (((l) & 0x000000ff) << 24))
#define SWAP_16(l) \
    ((((l) & 0xff00) >> 8) | \
     (((l) & 0x00ff) << 8))

#if defined(__BYTE_ORDER) && (__BYTE_ORDER == __BIG_ENDIAN)
#   define le64_to_cpu(x) SWAP_64(x)
#   define cpu_to_le64(x) SWAP_64(x)
#   define le32_to_cpu(x) SWAP_32(x)
#   define cpu_to_le32(x) SWAP_32(x)
#   define le16_to_cpu(x) SWAP_16(x)
#   define cpu_to_le16(x) SWAP_16(x)
#else
#   define le64_to_cpu(x) (x)
#   define cpu_to_le64(x) (x)
#   define le32_to_cpu(x) (x)
#   define cpu_to_le32(x) (x)
#   define le16_to_cpu(x) (x)
#   define cpu_to_le16(x) (x)
#endif // little endian

#if !defined(DOXYGEN_SHOULD_SKIP_THIS) && !defined(WRAP_ANY)
#   ifdef _WIN32
#       ifdef __BORLANDC__
#           pragma option push -b // force enums to the size of integer
#       endif // __BORLANDC__
#   endif // _WIN32
#endif // #if !defined(DOXYGEN_SHOULD_SKIP_THIS) && !defined(WRAP_ANY)

#ifdef __GNUC__
#   define ATTR_PACK __attribute__((packed)) __attribute__ ((aligned (4)))
#elif defined _MSC_VER
#   define ATTR_PACK
#   pragma pack(push, 4) // 4 byte structure alignment
#else
#   error add your compiler specific structure alignment function here
#endif

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

//-----------------------------------------------------------------------------
enum TArraySizeConstants
//-----------------------------------------------------------------------------
{
    CHUNKCOUNTERSELECTOR_COUNT = 8,
    CHUNKTIMERSELECTOR_COUNT = 2,
    MVCHUNKJPEG_SIZE = 1024,
    CHUNKGAINSELECTOR_COUNT = 5
};

//-----------------------------------------------------------------------------
enum TChunkIDs
//-----------------------------------------------------------------------------
{
    cidImagePort = 0x11223344,
    cidImageInfoVersion1Port = 0x44455566,
    cidJPEGPort = 0x44455567,
    cidAdditionalInfoPort = 0x44455568,
    cidImageInfoVersion2Port = 0x44455569
};

//-----------------------------------------------------------------------------
/// \brief Returns the PixelFormat of the image included in the payload.
///
/// Returns the PixelFormat of the image included in the payload.
enum TChunkPixelFormat
//-----------------------------------------------------------------------------
{
    /// \brief Mono 8 bit packed.
    ///
    /// Mono 8 bit packed.
    cpfMono8 = 0x01080001,
    /// \brief Mono 10 bit.
    ///
    /// Mono 10 bit.
    cpfMono10 = 0x01100003,
    /// \brief Mono 12 bit packed.
    ///
    /// Mono 12 bit packed.
    cpfMono12 = 0x01100005,
    /// \brief Mono 14 bit.
    ///
    /// Mono 14 bit.
    cpfMono14 = 0x01100025,
    /// \brief Mono 16 bit.
    ///
    /// Mono 16 bit.
    cpfMono16 = 0x01100007,
    /// \brief Mono 12 bit packed (GigE Vision Specific).
    ///
    /// Mono 12 bit packed (GigE Vision Specific).
    cpfMono12Packed = 0x010C0006,
    /// \brief Bayer Green Red 8 bit.
    ///
    /// Bayer Green Red 8 bit.
    cpfBayerGR8 = 0x01080008,
    /// \brief Bayer Red Green 8 bit.
    ///
    /// Bayer Red Green 8 bit.
    cpfBayerRG8 = 0x01080009,
    /// \brief Bayer Green Blue 8 bit.
    ///
    /// Bayer Green Blue 8 bit.
    cpfBayerGB8 = 0x0108000A,
    /// \brief Bayer Blue Green 8 bit.
    ///
    /// Bayer Blue Green 8 bit.
    cpfBayerBG8 = 0x0108000B,
    /// \brief Bayer GR 12 bit packed (GigE Vision Specific).
    ///
    /// Bayer GR 12 bit packed (GigE Vision Specific).
    cpfBayerGR12Packed = 0x010C002A,
    /// \brief Bayer RG 12 bit packed (GigE Vision Specific).
    ///
    /// Bayer RG 12 bit packed (GigE Vision Specific).
    cpfBayerRG12Packed = 0x010C002B,
    /// \brief Bayer GB 12 bit packed (GigE Vision Specific).
    ///
    /// Bayer GB 12 bit packed (GigE Vision Specific).
    cpfBayerGB12Packed = 0x010C002C,
    /// \brief Bayer BG 12 bit packed (GigE Vision Specific).
    ///
    /// Bayer BG 12 bit packed (GigE Vision Specific).
    cpfBayerBG12Packed = 0x010C002D,
    /// \brief Bayer Green Red 10 bit.
    ///
    /// Bayer Green Red 10 bit.
    cpfBayerGR10 = 0x0110000C,
    /// \brief Bayer Red Green 10 bit.
    ///
    /// Bayer Red Green 10 bit.
    cpfBayerRG10 = 0x0110000D,
    /// \brief Bayer Green Blue 10 bit.
    ///
    /// Bayer Green Blue 10 bit.
    cpfBayerGB10 = 0x0110000E,
    /// \brief Bayer Blue Green 10 bit.
    ///
    /// Bayer Blue Green 10 bit.
    cpfBayerBG10 = 0x0110000F,
    /// \brief Bayer Green Red 12 bit.
    ///
    /// Bayer Green Red 12 bit.
    cpfBayerGR12 = 0x01100010,
    /// \brief Bayer Red Green 12 bit.
    ///
    /// Bayer Red Green 12 bit.
    cpfBayerRG12 = 0x01100011,
    /// \brief Bayer Green Blue 12 bit.
    ///
    /// Bayer Green Blue 12 bit
    cpfBayerGB12 = 0x01100012,
    /// \brief Bayer Blue Green 12 bit.
    ///
    /// Bayer Blue Green 12 bit
    cpfBayerBG12 = 0x01100013,
    /// \brief Bayer Green Red 16 bit.
    ///
    /// Bayer Green Red 16 bit.
    cpfBayerGR16 = 0x0110002E,
    /// \brief Bayer Red Green 16 bit.
    ///
    /// Bayer Red Green 16 bit.
    cpfBayerRG16 = 0x0110002F,
    /// \brief Bayer Green Blue 16 bit.
    ///
    /// Bayer Green Blue 16 bit.
    cpfBayerGB16 = 0x01100030,
    /// \brief Bayer Blue Green 16 bit.
    ///
    /// Bayer Blue Green 16 bit.
    cpfBayerBG16 = 0x01100031,
    cpfRGB8Packed = 0x02180014,
    cpfBGR8Packed = 0x02180015,
    cpfBGRA8Packed = 0x02200017,
    /// \brief BGR 10 bit packed (GigE Vision Specific).
    ///
    /// BGR 10 bit packed (GigE Vision Specific).
    cpfBGR10V1Packed = 0x0220001C,
    cpfBGR10V2Packed = 0x0220001D,
    cpfRGB12Packed = 0x0230001A,
    cpfBGR12Packed = 0x0230001B,
    cpfYUV422Packed = 0x0210001F,
    cpfYUV422_YUYVPacked = 0x02100032,
    cpfYUV444Packed = 0x02180020,
    /// \brief Red, Green, Blue 8 bit.
    ///
    /// Red, Green, Blue 8 bit
    cpfRGB8 = 0,
    /// \brief Blue, Green, Red 8 bit.
    ///
    /// Blue, Green, Red 8 bit.
    cpfBGR8 = 1,
    /// \brief Blue, Green, Red, Alpha 8 bit.
    ///
    /// Blue, Green, Red, Alpha 8 bit.
    cpfBGRa8 = 2,
    /// \brief Red, Green, Blue 10 bit packed in 32 bit pixel.
    ///
    /// Red, Green, Blue 10 bit packed in 32 bit pixel.
    cpfRGB10p32 = 3,
    /// \brief Red, Green, Blue 12 bit.
    ///
    /// Red, Green, Blue 12 bit.
    cpfRGB12 = 4,
    /// \brief Blue, Green, Red 12 bit.
    ///
    /// Blue, Green, Red 12 bit.
    cpfBGR12 = 5,
    cpfYUV422_8_UYVY = 6,
    /// \brief YUV 422 8 bit.
    ///
    /// YUV 422 8 bit.
    cpfYUV422_8 = 7,
    cpfYUV8_UYV = 8,
    cpfYUV411_8_UYYVYY = 0x020C001E,
    cpfMono12p = 0x010c0047,
    cpfBayerGR12p = 0x010C0057,
    cpfBayerRG12p = 0x010C0059,
    cpfBayerGB12p = 0x010C0055,
    cpfBayerBG12p = 0x010C0053
};

//-----------------------------------------------------------------------------
enum TStringSizeConstants
//-----------------------------------------------------------------------------
{
    CHUNKADDITIONALINFO_SIZE = 64
};

//-----------------------------------------------------------------------------
/// \brief A structure modeling the IMAGE_INFO_CHUNK_RO_Segment (version 1: used with firmware version <= 2.13) segment.
///
/// This structure is 80(0x00000050) bytes large.
/// This structure does define the memory layout of the chunk '44455566'.
typedef struct _IMAGE_INFO_VERSION1_CHUNK_RO_Segment
//-----------------------------------------------------------------------------
{
    /// \brief Returns the OffsetX of the image included in the payload.
    ///
    /// Returns the OffsetX of the image included in the payload.
    mv_int32 ChunkOffsetX; // offset: 0x00000000
    /// \brief Returns the OffsetY of the image included in the payload.
    ///
    /// Returns the OffsetY of the image included in the payload.
    mv_int32 ChunkOffsetY; // offset: 0x00000004
    /// \brief Returns the Width of the image included in the payload.
    ///
    /// Returns the Width of the image included in the payload.
    mv_int32 ChunkWidth; // offset: 0x00000008
    /// \brief Returns the Height of the image included in the payload.
    ///
    /// Returns the Height of the image included in the payload.
    mv_int32 ChunkHeight; // offset: 0x0000000c
    /// \brief Returns the PixelFormat of the image included in the payload.
    ///
    /// Returns the PixelFormat of the image included in the payload.
    TChunkPixelFormat ChunkPixelFormat; // offset: 0x00000010
    /// \brief Returns the Timestamp of the image included in the payload at the time of the FrameStart internal event.
    ///
    /// Returns the Timestamp of the image included in the payload at the time of the FrameStart internal event.
    mv_uint64 ChunkTimestamp; // offset: 0x00000014
    /// \brief Returns the status of all the I/O lines at the time of the FrameStart internal event.
    ///
    /// Returns the status of all the I/O lines at the time of the FrameStart internal event.
    mv_int32 ChunkLineStatusAll; // offset: 0x0000001c
    /// \brief Returns the value of the selected Chunk counter at the time of the FrameStart event.
    ///
    /// Returns the value of the selected Chunk counter at the time of the FrameStart event.
    mv_int32 ChunkCounterValue[CHUNKCOUNTERSELECTOR_COUNT]; // offset: 0x00000020
    /// \brief Returns the value of the selected Timer at the time of the FrameStart internal event.
    ///
    /// Returns the value of the selected Timer at the time of the FrameStart internal event.
    mv_int32 ChunkTimerValue[CHUNKTIMERSELECTOR_COUNT]; // offset: 0x00000040
    /// \brief Returns the exposure time used to capture the image.
    ///
    /// Returns the exposure time used to capture the image.
    mv_int32 ChunkExposureTime; // offset: 0x00000048
    /// \brief Return the index of the active set of the running sequencer included in the payload.
    ///
    /// Return the index of the active set of the running sequencer included in the payload.
    mv_int32 ChunkSequencerSetActive; // offset: 0x0000004c
} ATTR_PACK IMAGE_INFO_VERSION1_CHUNK_RO_Segment;

//-----------------------------------------------------------------------------
/// \brief A structure modeling the JPEG_CHUNK_RO_Segment segment.
///
/// This structure is 1028(0x00000404) bytes large.
/// This structure does define the memory layout of the chunk '44455567'.
typedef struct _JPEG_CHUNK_RO_Segment
//-----------------------------------------------------------------------------
{
    /// \brief JPEG image from FPGA
    ///
    /// JPEG image from FPGA
    mv_uint32 mvChunkJPEGLength_InqLength; // offset: 0x00000000
    /// \brief JPEG image from FPGA
    ///
    /// JPEG image from FPGA
    char mvChunkJPEG[MVCHUNKJPEG_SIZE]; // offset: 0x00000004
} ATTR_PACK JPEG_CHUNK_RO_Segment;

//-----------------------------------------------------------------------------
/// \brief A structure modeling the ADDITIONAL_INFO_CHUNK_RO_Segment segment.
///
/// This structure is 64(0x00000040) bytes large.
/// This structure does define the memory layout of the chunk '44455568'.
typedef struct _ADDITIONAL_INFO_CHUNK_RO_Segment
//-----------------------------------------------------------------------------
{
    /// \brief Data from external device
    ///
    /// Data from external device
    char ChunkAdditionalInfo[CHUNKADDITIONALINFO_SIZE]; // offset: 0x00000000
} ATTR_PACK ADDITIONAL_INFO_CHUNK_RO_Segment;

//-----------------------------------------------------------------------------
/// \brief A structure modeling the IMAGE_INFO_CHUNK_RO_Segment (version 2: used with firmware >= 2.14) segment.
///
/// \since 2.14.0
/// This structure is 104(0x00000068) bytes large.
/// This structure does define the memory layout of the chunk '44455569'.
typedef struct _IMAGE_INFO_VERSION2_CHUNK_RO_Segment
//-----------------------------------------------------------------------------
{
    /// \brief Returns the OffsetX of the image included in the payload.
    ///
    /// Returns the OffsetX of the image included in the payload.
    mv_int32 ChunkOffsetX; // offset: 0x00000000
    /// \brief Returns the OffsetY of the image included in the payload.
    ///
    /// Returns the OffsetY of the image included in the payload.
    mv_int32 ChunkOffsetY; // offset: 0x00000004
    /// \brief Returns the Width of the image included in the payload.
    ///
    /// Returns the Width of the image included in the payload.
    mv_int32 ChunkWidth; // offset: 0x00000008
    /// \brief Returns the Height of the image included in the payload.
    ///
    /// Returns the Height of the image included in the payload.
    mv_int32 ChunkHeight; // offset: 0x0000000c
    /// \brief Returns the PixelFormat of the image included in the payload.
    ///
    /// Returns the PixelFormat of the image included in the payload.
    TChunkPixelFormat ChunkPixelFormat; // offset: 0x00000010
    /// \brief Returns the Timestamp of the image included in the payload at the time of the FrameStart internal event.
    ///
    /// Returns the Timestamp of the image included in the payload at the time of the FrameStart internal event.
    mv_uint64 ChunkTimestamp; // offset: 0x00000014
    /// \brief Returns the status of all the I/O lines at the time of the FrameStart internal event.
    ///
    /// Returns the status of all the I/O lines at the time of the FrameStart internal event.
    mv_int32 ChunkLineStatusAll; // offset: 0x0000001c
    /// \brief Returns the value of the selected Chunk counter at the time of the FrameStart event.
    ///
    /// Returns the value of the selected Chunk counter at the time of the FrameStart event.
    mv_int32 ChunkCounterValue[CHUNKCOUNTERSELECTOR_COUNT]; // offset: 0x00000020
    /// \brief Returns the value of the selected Timer at the time of the FrameStart internal event.
    ///
    /// Returns the value of the selected Timer at the time of the FrameStart internal event.
    mv_int32 ChunkTimerValue[CHUNKTIMERSELECTOR_COUNT]; // offset: 0x00000040
    /// \brief Returns the exposure time used to capture the image.
    ///
    /// Returns the exposure time used to capture the image.
    mv_int32 ChunkExposureTime; // offset: 0x00000048
    /// \brief Return the index of the active set of the running sequencer included in the payload.
    ///
    /// Return the index of the active set of the running sequencer included in the payload.
    mv_int32 ChunkSequencerSetActive; // offset: 0x0000004c
    /// \brief Returns the gain used to capture the image.
    ///
    /// Returns the gain used to capture the image.
    mv_int32 ChunkGain[CHUNKGAINSELECTOR_COUNT]; // offset: 0x00000050
    /// \brief Returns an identifier provided by the user.
    ///
    /// Returns an identifier provided by the user.
    mv_int32 ChunkmvCustomIdentifier; // offset: 0x00000064
} ATTR_PACK IMAGE_INFO_VERSION2_CHUNK_RO_Segment;


#if !defined(DOXYGEN_SHOULD_SKIP_THIS) && !defined(WRAP_ANY)
typedef enum TArraySizeConstants TArraySizeConstants;
typedef enum TChunkIDs TChunkIDs;
typedef enum TChunkPixelFormat TChunkPixelFormat;
typedef enum TStringSizeConstants TStringSizeConstants;
#endif // DOXYGEN_SHOULD_SKIP_THIS && WRAP_ANY

#ifdef __cplusplus
}
#endif // __cplusplus

#ifdef __GNUC__
#   undef ATTR_PACK
#elif defined _MSC_VER
#   undef ATTR_PACK
#   pragma pack(pop) // restore previous structure alignment
#else
#   error restore your compiler specific structure alignment here if necessary
#endif

// restore Borland compiler switch 'force enums to the size of integer'
#if !defined(DOXYGEN_SHOULD_SKIP_THIS) && !defined(WRAP_ANY)
#   ifdef _WIN32
#       ifdef __BORLANDC__
#           pragma option pop
#       endif // __BORLANDC__
#   endif // _WIN32
#endif // !defined(DOXYGEN_SHOULD_SKIP_THIS) && !defined(WRAP_ANY)

#endif // CHUNK_LAYOUTS_H
